"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.api = void 0;
const core_1 = require("@angular-devkit/core");
const schematics_1 = require("@angular-devkit/schematics");
const chalk = require("chalk");
const sync_request_1 = require("sync-request");
const R = require("ramda");
const PATH = require("path");
const log = console.log;
function api(options) {
    const { old, source_path, total } = options;
    const list = getApiList(options);
    const path = PATH.basename(process.cwd() + source_path);
    const filePath = core_1.join(path);
    return (tree, context) => {
        const ruleList = [];
        const optionsList = [];
        list.map(item => {
            const option = transform(options, item, path);
            ruleList.push(schematics_1.mergeWith(generate(option, path, tree), schematics_1.MergeStrategy.Overwrite));
            optionsList.push(option);
        });
        return schematics_1.branchAndMerge(schematics_1.chain([
            ...ruleList,
            cleanPath(filePath),
            addToApi(optionsList, filePath, options, old),
            updateConfig(options)
        ]))(tree, context);
    };
}
exports.api = api;
const fetchYAPiData = (url, yapiCookie) => {
    const res = sync_request_1.default("GET", url, {
        headers: {
            'Cookie': yapiCookie
        }
    });
    if (res) {
        try {
            const result = JSON.parse(res.getBody('utf8'));
            return result;
        }
        catch (error) {
            log(chalk.red('»» api.factory.ts schematics fetchYAPiData Error=>\r\n'), error);
            return null;
        }
    }
    else {
        return null;
    }
};
const getApiList = (options) => {
    const { yapi_domain, yapiCookie, yapi_apis } = options;
    const apiResult = [];
    const fetchApi = ({ id, projectId, isFunction, isMethod, isV2, isMock }) => {
        const result = fetchYAPiData(`${yapi_domain}api/interface/get?id=${id}`, JSON.parse(decodeURIComponent(yapiCookie)));
        if (result) {
            if (result.errcode > 0 || !result.data) {
                log(chalk.red('✖️ api.factory 从YAPI获取API定义发生错误：'), `API: ${id} => ${result.errmsg}`);
            }
            else {
                apiResult.push({
                    api: result.data,
                    projectId,
                    projectInfo: result.data.projectInfo,
                    isFunction,
                    isMethod,
                    isMock,
                    isV2,
                });
            }
        }
    };
    if (yapi_apis) {
        const apis = JSON.parse(decodeURI(yapi_apis || '{}'));
        apis.map((item) => {
            fetchApi(item);
        });
    }
    return apiResult;
};
const transform = (source, data, path) => {
    const target = R.clone(source);
    const { api, projectId, isFunction, isMethod, projectInfo, isV2, isMock } = data;
    const finalPath = projectInfo.basepath !== '' ? `${projectInfo.basepath}/${api.path}` : api.path;
    api.path = finalPath.replace(new RegExp("\/\/", "gm"), "/");
    const isOld = target.old;
    let name = '';
    if (isOld) {
        name = transformToName(api.path, isOld);
    }
    else {
        name = transformToName(api._id, isOld);
    }
    let inputs = [];
    let requestRes = { responses: [], interfaces: undefined };
    if (api.req_query && api.req_query.length) {
        inputs = reduceQuery(api.req_query);
    }
    else if (api.req_params && api.req_params.length) {
        inputs = reduceRequestParams(api.req_params);
    }
    else if (api.req_body_form && api.req_body_form.length || api.req_body_other) {
        inputs = reduceRequestBodyForm(api.req_body_form);
        if (api.req_body_other) {
            requestRes = reduceResponse(api.req_body_other, `${name}Req`, api.method, isMethod);
            inputs = [
                ...inputs,
                ...reduceRequestBodyOther(requestRes.responses)
            ];
        }
        else {
        }
    }
    const responsesRes = api.res_body ? reduceResponse(api.res_body, `${name}Res`, api.method, isMethod) : {};
    if (requestRes.interfaces) {
        responsesRes.interfaces = Object.assign(Object.assign({}, requestRes.interfaces), responsesRes.interfaces);
    }
    const interfaces = responsesRes.interfaces ? R.pipe(R.keys, R.map(item => R.merge({ data: responsesRes.interfaces[item] }, { name: item })), R.flatten)(responsesRes.interfaces) : [];
    target.name = name;
    target.title = api.title;
    target.path = transformToPath(api.path);
    target.id = api._id;
    target.method = api.method;
    target.projectId = projectId.indexOf('_V2') > -1 ? projectId.replace('_V2', '') : projectId;
    target.project_id = api.project_id;
    target.inputs = inputs;
    target.requestType = requestRes.responsesType || '{}';
    target.responses = responsesRes.responses;
    target.responsesType = responsesRes.responsesType || '{}';
    target.interfaces = interfaces;
    target.isRestful = api.path.indexOf('{') >= 0 && api.path.indexOf('}') >= 0;
    target.filePath = core_1.join(path);
    target.isFunction = isFunction;
    target.isMethod = isMethod;
    target.isV2 = isV2;
    target.isMock = isMock;
    target.file_name = isOld ? api.method + name : name;
    return target;
};
const generate = (options, path, tree) => {
    return schematics_1.apply(schematics_1.url('./files'), [
        schematics_1.template(Object.assign(Object.assign({}, core_1.strings), options)),
        schematics_1.forEach((fileEntry) => {
            if (tree.exists(path + fileEntry.path)) {
                tree.overwrite(path + fileEntry.path, fileEntry.content);
            }
            ;
            return fileEntry;
        }),
        schematics_1.move(path),
    ]);
};
const typeMap = {
    'integer': 'number',
    'Integer': 'number',
    'int': 'number',
    'BigDecimal': 'number',
    'Long': 'number',
    'long': 'number',
    'number': 'number',
    'array': 'any[]',
    'Array': 'any[]',
    'String': 'string',
    'string': 'string',
    'Date': 'string',
    'date': 'string',
    'text': 'string',
    'boolean': 'boolean',
    'Boolean': 'boolean',
    'enum': 'enum',
    'file': 'any',
};
const transformType = (data) => {
    const type = typeMap[data.type] || 'any';
    if (type === 'enum') {
        return 'string';
    }
    else {
        return type;
    }
};
const transformToName = (str, isOld) => {
    if (isOld) {
        let result = str;
        result = result.replace('.html', '');
        result = result.replace('-', '');
        return result ? R.pipe(R.split('/'), R.map((item) => {
            item = item ? item.split('{').join('') : item;
            item = item ? item.split('}').join('') : item;
            return R.toUpper(item.charAt(0)) + R.toLower(item.substring(1));
        }), R.join(''))(result.replace('-', '')) : result.replace('-', '');
    }
    else {
        return `API${str}`;
    }
};
const transformToPath = (str) => {
    return str ? R.pipe(R.split('/'), R.filter((item) => item.indexOf('{') < 0 && item.indexOf('}') < 0), R.join('/'))(str) : str;
};
const reduceRequestParams = (list) => {
    return R.map((item) => `
    ${item.name}: string;
  `)(list);
};
const reduceQuery = (list) => {
    const res = list && list.length ? R.pipe(R.flatten, R.map((item) => `
  /** ${item.desc || ''} */
  ${item.name}${item.required === '1' ? '' : '?'}: ${typeMap[item.example] ? typeMap[item.example] : 'string'};
    `))(list) : [];
    return res;
};
const reduceRequestBodyOther = (list) => {
    const reduceItem = (item) => {
        return `
  /** ${item.description || ''} */
  ${item.name}${item.required ? '' : '?'}: ${item.type};`;
    };
    return list && list.length ? R.map(item => reduceItem(item))(list) : [];
};
const reduceRequestBodyForm = (forms = []) => {
    if (!forms.length) {
        return forms;
    }
    const reduceItem = (item) => {
        return `
  /** ${item.desc || ''} */
  ${item.name}${item.required && item.required === '1' ? '' : '?'}: ${reduceType(item)};`;
    };
    return forms && forms.length ? R.map(item => reduceItem(item))(forms) : [];
};
const reduceType = (data) => {
    if (!data.type) {
        return 'any';
    }
    else {
        return transformType(data) || 'any';
    }
};
const reduceResponse = (str, name, method, isMethod) => {
    var _a;
    if (str.indexOf('{') < 0) {
        return {
            responses: [],
            interfaces: {}
        };
    }
    const res = JSON.parse(str);
    const interfaceName = `I${isMethod ? method : ''}${name}`;
    if (res.properties && res.properties.data && res.properties.data.type === 'object') {
        const { properties, required = {} } = ((_a = res.properties) === null || _a === void 0 ? void 0 : _a.data) || {};
        return transformResponse(properties, required, interfaceName);
    }
    else if (res.properties && res.properties.data && res.properties.data.type !== 'object') {
        const { properties, required = {} } = res || {};
        const result = transformResponse({ data: properties === null || properties === void 0 ? void 0 : properties.data }, required, interfaceName);
        return {
            responses: [],
            interfaces: result.interfaces,
            responsesType: result.responses[0].type,
        };
    }
    else if (res.properties && !res.properties.data) {
        const { properties, required = {} } = res;
        return transformResponse(properties, required, interfaceName);
    }
    else {
        const result = transformResponse({ data: res }, [], interfaceName);
        return {
            responses: [],
            interfaces: result.interfaces,
            responsesType: result.responses[0].type,
        };
    }
};
const transformResponse = (data, required, name, interfaces = {}) => {
    const responses = [];
    const keys = R.keys(data);
    keys.map((key) => {
        let interfaceName = `${name}${UpperCaseParam(key)}`;
        let type;
        if (data[key].type === 'array') {
            interfaceName = `${interfaceName}Item`;
            type = reduceArray(data[key], interfaceName);
        }
        else if (data[key].type === 'object') {
            interfaceName = `${interfaceName}Obj`;
            type = interfaceName;
        }
        else {
            type = reduceType(data[key]);
        }
        responses.push({
            name: key,
            type,
            required: checkRequire(required, key),
            description: data[key].description,
        });
        if (data[key].items || data[key].type === 'object') {
            const { items, type } = data[key];
            const { properties, required } = type === 'object' ? data[key] : findProperties(items);
            if (!interfaces[interfaceName] && properties) {
                interfaces[interfaceName] = transformResponse(properties, required, interfaceName, interfaces).responses;
            }
        }
    });
    return {
        responses,
        interfaces
    };
};
const UpperCaseParam = (name) => {
    return name.slice(0, 1).toUpperCase() + name.slice(1);
};
const findProperties = (data) => {
    const { type } = data;
    if (type === 'object') {
        return { properties: data.properties, required: data.required };
    }
    if (type === 'array') {
        return findProperties(data.items);
    }
    else {
        return {};
    }
};
const checkRequire = (required, key) => {
    if (!required) {
        return false;
    }
    if (R.type(required) === 'Object') {
        return required[key] ? true : false;
    }
    else {
        return required.indexOf(key) >= 0 ? true : false;
    }
};
const reduceArray = (data, name) => {
    const { type } = data.items;
    if (type === 'object') {
        return `${name}[]`;
    }
    else if (type === 'array') {
        return `${reduceArray(data.items, name)}[]`;
    }
    else {
        return `${(typeMap[type]) || 'any'}[]`;
    }
};
const addToApi = (optionsList, filePath, options, old) => {
    return (tree) => {
        try {
            const outApiFile = `${filePath}/generated/api.ts`;
            const { new_apiId, new_apiId_method, new_apiId_function } = options;
            const hasNewApi = (new_apiId || new_apiId_method || new_apiId_function) ? true : false;
            let newContent = '';
            const hasFile = tree.exists(outApiFile);
            if (hasFile) {
                newContent = tree.read(outApiFile).toString();
            }
            optionsList.map(option => {
                if (newContent.indexOf(option.file_name) < 0 || newContent.indexOf(option.id) < 0) {
                    newContent += `
// id=${option.id} title=${option.title} path=${option.path} method=${option.method}
export * from './${option.file_name}';`;
                }
            });
            if (hasFile) {
                tree.overwrite(outApiFile, `${newContent}`);
            }
            else {
                tree.create(outApiFile, newContent);
            }
            return tree;
        }
        catch (error) {
            console.log('addToApi Error:', error);
        }
    };
};
const createUseApis = (optionsList, filePath, isOld) => {
    const importMap = {
        'POST': 'Post',
        'GET': 'Get'
    };
    const methodMap = {
        'POST': isOld ? 'post' : 'Post',
        'GET': isOld ? 'get' : 'Get'
    };
    return (tree) => {
        let imports = '';
        let methods = '';
        let results = '';
        const apiPath = filePath + '/generated/useApis.ts';
        let hasFile;
        let initContent = '';
        if (tree.exists(apiPath)) {
            const data = tree.read(apiPath).toString();
            if (data.indexOf("import {") < 0) {
                tree.delete(apiPath);
                hasFile = false;
            }
            else {
                initContent = data;
                hasFile = true;
            }
        }
        optionsList.map(option => {
            if (!option.isFunction) {
                const useName = `use${importMap[option.method]}${option.name}`;
                if (initContent.indexOf(useName) < 0) {
                    let methodName = '';
                    if (isOld) {
                        methodName = `${methodMap[option.method]}${option.name}Data`;
                    }
                    else {
                        methodName = `fetch${option.name}Data`;
                    }
                    imports += useName + ', \n';
                    methods += `const { ${methodName} } = ${useName}();\n`;
                    results += `id${option.id}: ${methodName},\n`;
                }
            }
        });
        if (hasFile) {
            const endImport = initContent.indexOf("} from './api';");
            initContent = insertStr(initContent, endImport, imports);
            const endHook = initContent.indexOf("return {");
            initContent = insertStr(initContent, endHook, methods);
            const endReturn = initContent.indexOf("}}");
            initContent = insertStr(initContent, endReturn, results);
            tree.overwrite(apiPath, initContent);
        }
        else {
            const content = `
      import {
          ${imports}
      } from './api';

      export const useApis = () => {
        ${methods}
        return {
          ${results}
      }}
          `;
            tree.create(apiPath, content);
        }
        return tree;
    };
};
function insertStr(str, start, newStr) {
    return str.slice(0, start) + newStr + str.slice(start);
}
const cleanPath = (filePath) => {
    return (tree) => {
        try {
            if (tree.exists(`${filePath}/.DS_Store`)) {
                tree.delete(`${filePath}/.DS_Store`);
            }
            if (tree.exists(`${filePath}/generated/.DS_Store`)) {
                tree.delete(`${filePath}/generated/.DS_Store`);
            }
        }
        catch (error) {
            console.error('cleanPath Error:', error);
        }
        return tree;
    };
};
const updateConfig = (options) => {
    const { yapi_apiId, yapi_apiId_function, yapi_apiId_method, new_apiId, new_apiId_method, new_apiId_function } = options;
    const configPath = `yw.config.json`;
    return (tree) => {
        if (tree.exists(configPath)) {
            const data = tree.read(configPath) || '{}';
            const result = JSON.parse(data.toString());
            const apiId = JSON.parse(decodeURI(yapi_apiId || new_apiId || '{}'));
            const apiId_function = JSON.parse(decodeURI(yapi_apiId_function || new_apiId_function || '{}'));
            const apiId_method = JSON.parse(decodeURI(yapi_apiId_method || new_apiId_method || '{}'));
            const concatValues = (k, l, r) => {
                return R.union(l, r);
            };
            result.yapi_apiId = R.mergeWithKey(concatValues, result.yapi_apiId, apiId);
            result.yapi_apiId_function = R.mergeWithKey(concatValues, result.yapi_apiId_function, apiId_function);
            result.yapi_apiId_method = R.mergeWithKey(concatValues, result.yapi_apiId_method, apiId_method);
            tree.overwrite(configPath, JSON.stringify(result, null, 2));
        }
        else {
        }
        return tree;
    };
};
