const path = require('path');
const fs = require('fs');
const { babel } = require('@rollup/plugin-babel');
const { nodeResolve } = require('@rollup/plugin-node-resolve');
const commonjs = require('@rollup/plugin-commonjs');
const postcss = require('rollup-plugin-postcss');
const rollupPostcssLessLoader = require('rollup-plugin-postcss-webpack-alias-less-loader');
const less = require('less');
const json = require('@rollup/plugin-json');
const typescript = require('@rollup/plugin-typescript');
const clear = require('rollup-plugin-clear');
const progress = require('rollup-plugin-progress');
const replace = require('@rollup/plugin-replace');
const builtins = require("rollup-plugin-node-builtins");
const image = require('@rollup/plugin-image');
const smartAsset = require('rollup-plugin-smart-asset');
const lessToJs = require('less-vars-to-js');
const nodePolyfills = require('rollup-plugin-node-polyfills');

const NODE_MODULE_PATH = path.join(__dirname, '../node_modules');
const extensions = ['.js', '.jsx', '.ts', '.tsx'];

const resolveFile = function (filePath) {
  return path.join(__dirname, '..', filePath)
}

const isProd = process.env.NODE_ENV === 'production';
const themeVariables = lessToJs(fs.readFileSync(path.join(__dirname, '../src/styles/theme.less'), 'utf8'), {
  resolveVariables: true, stripPrefix: true
});

const processLess = function (context, payload) {
  return new Promise((resolve, reject) => {
    less.render({
      file: context
    }, function (err, result) {
      if (!err) {
        resolve(result);
      } else {
        reject(err);
      }
    });

    less.render(context, {})
      .then(function (output) {
        if (output && output.css) {
          resolve(output.css);
        } else {
          reject({})
        }
      },
        function (err) {
          reject(err)
        });

  })
}

module.exports = [
  {
    input: resolveFile('index.ts'),
    output: {
      file: 'lib/<%= name%>.umd.js',
      format: 'umd',
      name: '<%= umdName%>',
      globals: {
        'react': 'React',
        'react-dom': 'ReactDOM',
        'antd': 'antd',
        'moment': 'moment',
        '@ant-design/icons': 'icons',
        '@ant-design/pro-table': 'ProTable',
        '@ant-design/charts': 'charts',
        '@ywfe/request': 'YWRequest',
        'ppfish': 'ppfish',
        '@ywfe/router': 'YWRouter',
        '@ywfe/yw-design': 'YwDesign',
      }
    },
    external: [
      "react", 'react-dom',
      'antd', 'moment', '@ant-design/icons', '@ant-design/pro-table', '@ant-design/charts',
      '@ywfe/request', 'ppfish', '@ywfe/router', '@ywfe/yw-design'
    ],
    inlineDynamicImports: true,
    plugins: [
      clear({
        targets: ['lib'],
      }),
      progress({
        clearLine: false,
      }),
      builtins(),
      replace({
        'process.env.REACT_APP_ENV': JSON.stringify(process.env.REACT_APP_ENV || 'dev'),
        'process.env.NODE_ENV': JSON.stringify(process.env.NODE_ENV || 'development'),
        'global.': 'window.'
      }),
      postcss({
        extract: true,
        minimize: isProd,
        process: processLess,
        loaders: [
          rollupPostcssLessLoader({
            nodeModulePath: NODE_MODULE_PATH,
            aliases: {},
          })
        ],
        extensions: ['.css', '.scss', '.less'],
        use: [
          'sass',
          ['less', { javascriptEnabled: true, modifyVars: themeVariables }]
        ],
      }),
      typescript(),
      json(),
      nodeResolve({
        mainFields: ["jsnext", "preferBuiltins", "browser"],
        browser: true
      }),
      commonjs({ browser: true }),
      nodePolyfills(),
      babel({
        exclude: 'node_modules/**',
        extensions
      }),
      image(),
      smartAsset({
        url: "copy",
        keepImport: true,
        useHash: false,
        keepName: true,
        extensions: [".txt"]
      })
    ],
  },
]
